@@ -37,6 +37,9 @@
    cases) for that.  */
 
 #include "ruby.h"
+#ifdef HAVE_RUBY_ENCODING_H
+#include "ruby/encoding.h"
+#endif
 #include "gpgme.h"
 #include <errno.h>
 
@@ -121,6 +124,7 @@ static VALUE cEngineInfo,
   cSignature,
   cSigNotation,
   cTrustItem,
+  cRecipient,
   cDecryptResult,
   cVerifyResult,
   cSignResult,
@@ -666,6 +670,36 @@ rb_s_gpgme_set_locale (VALUE dummy, VALUE vctx, VALUE vcategory, VALUE vvalue)
   return LONG2NUM(err);
 }
 
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010400
+static VALUE
+rb_s_gpgme_set_pinentry_mode (VALUE dummy, VALUE vctx, VALUE vmode)
+{
+  gpgme_ctx_t ctx;
+  gpgme_error_t err;
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  err = gpgme_set_pinentry_mode (ctx, NUM2INT(vmode));
+  return LONG2NUM(err);
+}
+
+static VALUE
+rb_s_gpgme_get_pinentry_mode (VALUE dummy, VALUE vctx)
+{
+  gpgme_ctx_t ctx;
+  gpgme_pinentry_mode_t mode;
+
+  UNWRAP_GPGME_CTX(vctx, ctx);
+  if (!ctx)
+    rb_raise (rb_eArgError, "released ctx");
+
+  mode = gpgme_get_pinentry_mode (ctx);
+  return INT2FIX(mode);
+}
+#endif
+
 static VALUE
 rb_s_gpgme_op_keylist_start (VALUE dummy, VALUE vctx, VALUE vpattern,
 			     VALUE vsecret_only)
@@ -718,6 +752,16 @@ rb_s_gpgme_op_keylist_ext_start (VALUE dummy, VALUE vctx, VALUE vpattern,
   return LONG2NUM(err);
 }
 
+static VALUE
+utf8_str_new (const char *data)
+{
+    VALUE string = rb_str_new2 (data);
+#ifdef HAVE_RUBY_ENCODING_H
+    rb_enc_associate_index (string, rb_enc_find_index ("UTF-8"));
+#endif
+    return string;
+}
+
 static VALUE
 save_gpgme_key_attrs (VALUE vkey, gpgme_key_t key)
 {
@@ -739,7 +783,7 @@ save_gpgme_key_attrs (VALUE vkey, gpgme_key_t key)
   if (key->issuer_serial)
     rb_iv_set (vkey, "@issuer_serial", rb_str_new2 (key->issuer_serial));
   if (key->issuer_name)
-    rb_iv_set (vkey, "@issuer_name", rb_str_new2 (key->issuer_name));
+    rb_iv_set (vkey, "@issuer_name", utf8_str_new (key->issuer_name));
   if (key->chain_id)
     rb_iv_set (vkey, "@chain_id", rb_str_new2 (key->chain_id));
   rb_iv_set (vkey, "@owner_trust", INT2FIX(key->owner_trust));
@@ -771,18 +815,20 @@ save_gpgme_key_attrs (VALUE vkey, gpgme_key_t key)
   rb_iv_set (vkey, "@uids", vuids);
   for (user_id = key->uids; user_id; user_id = user_id->next)
     {
-      VALUE vuser_id = rb_class_new_instance(0, NULL, cUserID), vsignatures;
+      VALUE vuser_id, vsignatures;
+      gpgme_key_sig_t key_sig;
+
+      vuser_id = rb_class_new_instance(0, NULL, cUserID);
       rb_iv_set (vuser_id, "@revoked", INT2FIX(user_id->revoked));
       rb_iv_set (vuser_id, "@invalid", INT2FIX(user_id->invalid));
       rb_iv_set (vuser_id, "@validity", INT2FIX(user_id->validity));
-      rb_iv_set (vuser_id, "@uid", rb_str_new2 (user_id->uid));
-      rb_iv_set (vuser_id, "@name", rb_str_new2 (user_id->name));
-      rb_iv_set (vuser_id, "@comment", rb_str_new2 (user_id->comment));
-      rb_iv_set (vuser_id, "@email", rb_str_new2 (user_id->email));
+      rb_iv_set (vuser_id, "@name", utf8_str_new (user_id->name));
+      rb_iv_set (vuser_id, "@uid", utf8_str_new (user_id->uid));
+      rb_iv_set (vuser_id, "@comment", utf8_str_new (user_id->comment));
+      rb_iv_set (vuser_id, "@email", utf8_str_new (user_id->email));
 
       vsignatures = rb_ary_new ();
       rb_iv_set (vuser_id, "@signatures", vsignatures);
-      gpgme_key_sig_t key_sig;
       for (key_sig = user_id->signatures; key_sig; key_sig = key_sig->next)
 	{
 	  VALUE vkey_sig = rb_class_new_instance(0, NULL, cKeySig);
@@ -1450,7 +1496,8 @@ rb_s_gpgme_op_decrypt_result (VALUE dummy, VALUE vctx)
 {
   gpgme_ctx_t ctx;
   gpgme_decrypt_result_t result;
-  VALUE vresult;
+  gpgme_recipient_t recipient;
+  VALUE vresult, vrecipients;
 
   UNWRAP_GPGME_CTX(vctx, ctx);
   if (!ctx)
@@ -1462,6 +1509,17 @@ rb_s_gpgme_op_decrypt_result (VALUE dummy, VALUE vctx)
     rb_iv_set (vresult, "@unsupported_algorithm",
 	       rb_str_new2 (result->unsupported_algorithm));
   rb_iv_set (vresult, "@wrong_key_usage", INT2FIX(result->wrong_key_usage));
+  vrecipients = rb_ary_new ();
+  rb_iv_set (vresult, "@recipients", vrecipients);
+  for (recipient = result->recipients; recipient; recipient = recipient->next)
+    {
+      VALUE vrecipient = rb_class_new_instance (0, NULL, cRecipient);
+      rb_iv_set (vrecipient, "@pubkey_algo", INT2FIX(recipient->pubkey_algo));
+      rb_iv_set (vrecipient, "@keyid", rb_str_new2 (recipient->keyid));
+      rb_iv_set (vrecipient, "@status", UINT2NUM(recipient->status));
+      rb_ary_push (vrecipients, vrecipient);
+    }
+  rb_iv_set (vresult, "@file_name", rb_str_new2 (result->file_name));
   return vresult;
 }
 
@@ -1972,6 +2030,8 @@ Init_gpgme_n (void)
     rb_define_class_under (mGPGME, "UserID", rb_cObject);
   cKeySig =
     rb_define_class_under (mGPGME, "KeySig", rb_cObject);
+  cRecipient =
+    rb_define_class_under (mGPGME, "Recipient", rb_cObject);
   cDecryptResult =
     rb_define_class_under (mGPGME, "DecryptResult", rb_cObject);
   cVerifyResult =
@@ -2047,6 +2107,12 @@ Init_gpgme_n (void)
 			     rb_s_gpgme_set_keylist_mode, 2);
   rb_define_module_function (mGPGME, "gpgme_get_keylist_mode",
 			     rb_s_gpgme_get_keylist_mode, 1);
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010400
+  rb_define_module_function (mGPGME, "gpgme_set_pinentry_mode",
+			     rb_s_gpgme_set_pinentry_mode, 2);
+  rb_define_module_function (mGPGME, "gpgme_get_pinentry_mode",
+			     rb_s_gpgme_get_pinentry_mode, 1);
+#endif
   rb_define_module_function (mGPGME, "gpgme_set_passphrase_cb",
 			     rb_s_gpgme_set_passphrase_cb, 3);
   rb_define_module_function (mGPGME, "gpgme_get_passphrase_cb",
@@ -2625,4 +2691,18 @@ Init_gpgme_n (void)
   rb_define_const (mGPGME, "GPGME_ENCRYPT_NO_ENCRYPT_TO",
 		   INT2FIX(GPGME_ENCRYPT_NO_ENCRYPT_TO));
 #endif
+
+  /* These flags were added in 1.4.0. */
+#if defined(GPGME_VERSION_NUMBER) && GPGME_VERSION_NUMBER >= 0x010400
+  rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_DEFAULT",
+                   INT2FIX(GPGME_PINENTRY_MODE_DEFAULT));
+  rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_ASK",
+                   INT2FIX(GPGME_PINENTRY_MODE_ASK));
+  rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_CANCEL",
+                   INT2FIX(GPGME_PINENTRY_MODE_CANCEL));
+  rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_ERROR",
+                   INT2FIX(GPGME_PINENTRY_MODE_ERROR));
+  rb_define_const (mGPGME, "GPGME_PINENTRY_MODE_LOOPBACK",
+                   INT2FIX(GPGME_PINENTRY_MODE_LOOPBACK));
+#endif
 }
