@@ -37,10 +37,18 @@ describe GPGME::Ctx do
 
         GPGME::Ctx.new(:password => 'gpgme') do |ctx|
           ctx.decrypt_verify input, output
-        end
 
-        output.seek 0
-        assert_equal "Hi there", output.read.chomp
+          output.seek 0
+          assert_equal "Hi there", output.read.chomp
+
+          recipients = ctx.decrypt_result.recipients
+          assert_equal 1, recipients.size
+
+          recipient_key = ctx.get_key(recipients.first.keyid)
+          key = ctx.get_key(PASSWORD_KEY[:sha])
+
+          assert_equal recipient_key, key
+        end
       end
     end
 
@@ -314,7 +322,7 @@ Key-Type: DSA
 Key-Length: 1024
 Subkey-Type: ELG-E
 Subkey-Length: 1024
-Name-Real: Key Tester
+Name-Real: Key Testér
 Name-Comment: with some comments
 Name-Email: test_generation@example.com
 Expire-Date: 0
@@ -322,6 +330,10 @@ Passphrase: wadus
 </GnupgKeyParms>
 RUBY
 
+      if RUBY_VERSION > "1.9"
+        assert_equal key.encoding, Encoding::UTF_8
+      end
+
       keys_amount = GPGME::Key.find(:public).size
       GPGME::Ctx.new do |ctx|
         ctx.generate_key(key.chomp)
@@ -330,6 +342,12 @@ RUBY
       assert_equal keys_amount + 1, GPGME::Key.find(:public).size
 
       GPGME::Key.find(:public, "test_generation@example.com").each do |k|
+
+        if RUBY_VERSION > "1.9"
+          # Make sure UTF-8 in and UTF-8 out.
+          assert_equal "Key Testér", k.name
+          assert_equal k.name.encoding, Encoding::UTF_8
+        end
         k.delete!(true)
       end
     end
